#include "ac_plugin.h"
#include "Undoables.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

//---------------------------------------------------------------------------------------
// Quick & dirty linked list for holding materials
class CDDHMatNode
{
public:
   CDDHMatNode *next;
   unsigned short idx;
   CDDHMatNode()
   {
      next = NULL;
   }
};
class CDDHMatList
{
public:
   CDDHMatNode *head;
   CDDHMatList()
   {
      head = NULL;
   }
   ~CDDHMatList()
   {
      CDDHMatNode *tnode;
      while( head != NULL )
      {
         tnode = head;
         head = head->next;
         delete tnode;
      }
   }
   void AddUnique( unsigned short s )
   {
      // Create a new item if none exist
      if( head == NULL )
      {
         head = new CDDHMatNode();
         head->idx = s;
         return;
      }
      // Find the item in the list
      CDDHMatNode *lnode = NULL;
      for( CDDHMatNode *n = head; n != NULL; n = n->next )
      {
         if( n->idx == s )
         {
            return;
         }
         lnode = n;
      }
      // Item not found - append it
      lnode->next = new CDDHMatNode();
      lnode->next->idx = s;
   }
};
//---------------------------------------------------------------------------------------
static void ddhselmat_addSurfsWithMat( List **l, ACObject *o, CDDHMatList &matlist )
{
   for( List *p = ac_object_get_childrenlist( o ); p != NULL; p = p->next )
   {
      ACObject *obj = ( ACObject * )p->data;
      if( ac_object_get_type( obj ) == OBJECT_NORMAL && ac_object_is_visible( obj ))
      {
         for( List *ps = ac_object_get_surfacelist( obj ); ps != NULL; ps = ps->next )
         {
            Surface *s = ( Surface * )( ps->data );
            for( CDDHMatNode *n = matlist.head; n != NULL; n = n->next )
            {
               if( n->idx == s->col )
               {
                  list_add_item_head( l, s );
               }
            }
         }
      }
      ddhselmat_addSurfsWithMat( l, obj, matlist );
   }
}
//---------------------------------------------------------------------------------------
// Select all materials with the given index
static void ddhselmat()
{

   // Grab all selected material indices
   List *selsurfs = ac_selection_get_whole_surfaces_all();
   CDDHMatList materials;
   for( List *p = selsurfs; p != NULL; p = p->next )
   {
      Surface *surf = ( Surface * )( p->data );
      materials.AddUnique( surf->col );
   }
   list_free( &selsurfs );

   // Append surfaces with like materials
   List *newsurfs = NULL;
   ddhselmat_addSurfsWithMat( &newsurfs, ac_get_world(), materials );

   // Select the surfaces
   add_undoable_change_selection( "select like materials" );
   ac_selection_select_by_surfaces( newsurfs );

   // Notification and cleanup
   int numsel = list_count( newsurfs );
   list_free( &newsurfs );
   redraw_all();
   display_status();
   display_message( "selmat : found %d surfaces", numsel );

}
//---------------------------------------------------------------------------------------
AC3D_PLUGIN_FUNC int AC3DPluginInit( AC3DPluginInitData *d )
{

   static bool firstTime = true;
   if( firstTime )
   {

      ac_add_command( "ddhselmat", ddhselmat );
      ac_add_tools_menu_item( "Select surfaces with like material", "ac3d ddhselmat",
                              "Select all objects with the same materials found in the current selection" );

      firstTime = false;

   }

   return 0;

}
//---------------------------------------------------------------------------------------
AC3D_PLUGIN_FUNC int AC3DPluginExit()
{
   return 0;
}
//---------------------------------------------------------------------------------------
AC3D_PLUGIN_FUNC char *AC3DPluginAbout()
{
   return( "Select Surfaces With Like Material - version 1.0.1 - Dennis Hawthorne" );
}
//---------------------------------------------------------------------------------------
AC3D_PLUGIN_FUNC char *AC3DPluginInfo()
{
   return( "Select Surfaces With Like Texture utility - Selects all surfaces which have the same material(s) found in the current selection." );
}
//---------------------------------------------------------------------------------------

